'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('../../utils');

var _ArrayField = require('./ArrayField');

var _ArrayField2 = _interopRequireDefault(_ArrayField);

var _BooleanField = require('./BooleanField');

var _BooleanField2 = _interopRequireDefault(_BooleanField);

var _NumberField = require('./NumberField');

var _NumberField2 = _interopRequireDefault(_NumberField);

var _ObjectField = require('./ObjectField');

var _ObjectField2 = _interopRequireDefault(_ObjectField);

var _StringField = require('./StringField');

var _StringField2 = _interopRequireDefault(_StringField);

var _UnsupportedField = require('./UnsupportedField');

var _UnsupportedField2 = _interopRequireDefault(_UnsupportedField);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var REQUIRED_FIELD_SYMBOL = '*';
var COMPONENT_TYPES = {
  array: _ArrayField2.default,
  boolean: _BooleanField2.default,
  integer: _NumberField2.default,
  number: _NumberField2.default,
  object: _ObjectField2.default,
  string: _StringField2.default
};

function getFieldComponent(schema, uiSchema, fields) {
  var field = uiSchema['ui:field'];
  if (typeof field === 'function') {
    return field;
  }
  if (typeof field === 'string' && field in fields) {
    return fields[field];
  }
  return COMPONENT_TYPES[schema.type] || _UnsupportedField2.default;
}

function Label(props) {
  var label = props.label,
      required = props.required,
      id = props.id;

  if (!label) {
    return null;
  }
  return _react2.default.createElement(
    'label',
    { className: 'control-label', htmlFor: id },
    required ? label + REQUIRED_FIELD_SYMBOL : label
  );
}

function Help(props) {
  var help = props.help;

  if (!help) {
    return null;
  }
  if (typeof help === 'string') {
    return _react2.default.createElement(
      'p',
      { className: 'help-block' },
      help
    );
  }
  return _react2.default.createElement(
    'div',
    { className: 'help-block' },
    help
  );
}

function ErrorList(props) {
  var _props$errors = props.errors,
      errors = _props$errors === undefined ? [] : _props$errors;

  if (errors.length === 0) {
    return null;
  }
  return _react2.default.createElement(
    'div',
    null,
    _react2.default.createElement('p', null),
    _react2.default.createElement(
      'ul',
      { className: 'error-detail bs-callout bs-callout-info' },
      errors.map(function (error, index) {
        return _react2.default.createElement(
          'li',
          { className: 'text-danger', key: index },
          error
        );
      })
    )
  );
}

function DefaultTemplate(props) {
  var id = props.id,
      classNames = props.classNames,
      label = props.label,
      children = props.children,
      errors = props.errors,
      help = props.help,
      description = props.description,
      hidden = props.hidden,
      required = props.required,
      displayLabel = props.displayLabel;

  if (hidden) {
    return children;
  }
  return _react2.default.createElement(
    'div',
    { className: classNames },
    displayLabel ? _react2.default.createElement(Label, { label: label, required: required, id: id }) : null,
    displayLabel && description ? description : null,
    children,
    errors,
    help
  );
}

if (process.env.NODE_ENV !== 'production') {
  DefaultTemplate.propTypes = {
    id: PropTypes.string,
    classNames: PropTypes.string,
    label: PropTypes.string,
    children: PropTypes.node.isRequired,
    errors: PropTypes.element,
    help: PropTypes.element,
    description: PropTypes.element,
    hidden: PropTypes.bool,
    required: PropTypes.bool,
    readonly: PropTypes.bool,
    displayLabel: PropTypes.bool,
    formContext: PropTypes.object
  };
}

DefaultTemplate.defaultProps = {
  hidden: false,
  readonly: false,
  required: false,
  displayLabel: true
};

function SchemaField(props) {
  var uiSchema = props.uiSchema,
      errorSchema = props.errorSchema,
      idSchema = props.idSchema,
      name = props.name,
      required = props.required,
      registry = props.registry;
  var definitions = registry.definitions,
      fields = registry.fields,
      formContext = registry.formContext,
      _registry$FieldTempla = registry.FieldTemplate,
      FieldTemplate = _registry$FieldTempla === undefined ? DefaultTemplate : _registry$FieldTempla;

  var schema = (0, _utils.retrieveSchema)(props.schema, definitions);
  var FieldComponent = getFieldComponent(schema, uiSchema, fields);
  var DescriptionField = fields.DescriptionField;

  var disabled = Boolean(props.disabled || uiSchema['ui:disabled']);
  var readonly = Boolean(props.readonly || uiSchema['ui:readonly']);

  if (Object.keys(schema).length === 0) {
    return _react2.default.createElement('div', null);
  }

  var displayLabel = true;
  if (schema.type === 'array') {
    displayLabel = (0, _utils.isMultiSelect)(schema) || (0, _utils.isFilesArray)(schema, uiSchema);
  }
  if (schema.type === 'object') {
    displayLabel = false;
  }
  if (schema.type === 'boolean' && !uiSchema['ui:widget']) {
    displayLabel = false;
  }
  if (uiSchema['ui:field']) {
    displayLabel = false;
  }

  var field = _react2.default.createElement(FieldComponent, _extends({}, props, {
    schema: schema,
    disabled: disabled,
    readonly: readonly,
    formContext: formContext
  }));

  var type = schema.type;

  var id = idSchema.$id;
  var label = props.schema.title || schema.title || name;
  var description = props.schema.description || schema.description;
  var errors = errorSchema.__errors;
  var help = uiSchema['ui:help'];
  var hidden = uiSchema['ui:widget'] === 'hidden';
  var classNames = ['form-group', 'field', 'field-' + type, errors && errors.length > 0 ? 'field-error has-error' : '', uiSchema.classNames].join(' ').trim();

  var fieldProps = {
    description: _react2.default.createElement(DescriptionField, {
      id: id + '__description',
      description: description,
      formContext: formContext
    }),
    help: _react2.default.createElement(Help, { help: help }),
    errors: _react2.default.createElement(ErrorList, { errors: errors }),
    id: id,
    label: label,
    hidden: hidden,
    required: required,
    readonly: readonly,
    displayLabel: displayLabel,
    classNames: classNames,
    formContext: formContext
  };

  return _react2.default.createElement(
    FieldTemplate,
    fieldProps,
    field
  );
}

SchemaField.defaultProps = {
  uiSchema: {},
  errorSchema: {},
  idSchema: {},
  registry: (0, _utils.getDefaultRegistry)(),
  disabled: false,
  readonly: false
};

exports.default = SchemaField;